import { AxiosPromise, AxiosRequestConfig } from 'axios';
import BaseDataMapper from "./base_data_mapper";
import DataRow from "./data_row";
import GetItemsResult from "./get_items_result";
export default abstract class RemoteDataMapper<T extends DataRow> extends BaseDataMapper<T> {
    protected abstract get url(): string;
    /**
     * Execute DELETE web method
     * @param key - resource key
     */
    delete(key: string): Promise<boolean>;
    /**
     * Execute GET web method for single resource
     * @param key - resource key
     */
    getItem(key: string): Promise<T | null>;
    /**
     * Execute GET web method for resources
     * @param filter - query filter
     */
    getItems(filter?: {
        params?: any;
    }): Promise<GetItemsResult<T>>;
    /**
     * Execute POST web method for new resource
     * @param payload - new resource data
     */
    insert(payload: {
        data: any;
    }): Promise<T>;
    /**
     * Execute PUT web method for update resource
     * @param key - resource key
     * @param payload - new resource data
     */
    update(key: string, payload: {
        data: any;
    }): Promise<boolean>;
    protected sendRequest(requestConfig: AxiosRequestConfig): AxiosPromise;
}
